package com.xkrs.straw.model.entity;

import javax.persistence.*;
import java.io.Serializable;

/**
 * 审核通过的精准火点
 */
@Entity
@Table(name = "fire_point_precise")
public class FirePointPreciseEntity implements Serializable {

    /**
     * 指定主键，建立自增序列，主键值取自序列
     */
    @Id
    @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "fire_point_precise_seq_gen")
    @SequenceGenerator(name = "fire_point_precise_seq_gen", sequenceName = "fire_point_precise_id_seq", allocationSize = 1)
    private Long id;

    /**
     * 火点编码
     */
    @Column(length = 64, unique = true, columnDefinition = "varchar(64)")
    private String fireCode;

    /**
     * 卫星监测的时间
     */
    @Column(length = 64, columnDefinition = "varchar(64)")
    private String satelliteTime;

    /**
     * 经度
     */
    private Double longitude;

    /**
     * 纬度
     */
    private Double latitude;

    /**
     * 卫星类型
     */
    @Column(length = 64, columnDefinition = "varchar(64)")
    private String satelliteType;

    /**
     * 植被类型
     */
    @Column(length = 64, columnDefinition = "varchar(64)")
    private String landType;

    /**
     * 置信度
     */
    @Column(length = 32, columnDefinition = "varchar(32)")
    private String confidence;

    /**
     * 当前火点的图片
     */
    @Column(length = 512, columnDefinition = "varchar(512)")
    private String fireImage;

    /**
     * 卫星影像图片
     */
    @Column(length = 512, columnDefinition = "varchar(512)")
    private String satelliteImage;

    /**
     * 添加的时间
     */
    @Column(length = 64, columnDefinition = "varchar(64)")
    private String addTime;

    /**
     * 详细地址
     */
    @Column(length = 255, columnDefinition = "varchar(255)")
    private String firePointAddress;

    /**
     * 备注
     */
    @Column(length = 1024, columnDefinition = "varchar(1024)")
    private String remark;

    /**
     * 占位符
     */
    @Column(length = 1024, columnDefinition = "varchar(1024)")
    private String placeHolder;

    /**
     * 省的编码
     */
    @Column(length = 64, columnDefinition = "varchar(64)")
    private String proCode;

    /**
     * 省的名称
     */
    @Column(length = 128, columnDefinition = "varchar(128)")
    private String proName;

    /**
     * 市的编码
     */
    @Column(length = 64, columnDefinition = "varchar(64)")
    private String cityCode;

    /**
     * 市的名称
     */
    @Column(length = 128, columnDefinition = "varchar(128)")
    private String cityName;

    /**
     * 区县的编码
     */
    @Column(length = 64, columnDefinition = "varchar(64)")
    private String countyCode;

    /**
     * 区县的名称
     */
    @Column(length = 128, columnDefinition = "varchar(128)")
    private String countyName;

    /**
     * 乡镇街道的编码
     */
    @Column(length = 64, columnDefinition = "varchar(64)")
    private String townCode;

    /**
     * 乡镇街道的名称
     */
    @Column(length = 128, columnDefinition = "varchar(128)")
    private String townName;

    public FirePointPreciseEntity() {
    }

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getFireCode() {
        return fireCode;
    }

    public void setFireCode(String fireCode) {
        this.fireCode = fireCode;
    }

    public String getSatelliteTime() {
        return satelliteTime;
    }

    public void setSatelliteTime(String satelliteTime) {
        this.satelliteTime = satelliteTime;
    }

    public Double getLongitude() {
        return longitude;
    }

    public void setLongitude(Double longitude) {
        this.longitude = longitude;
    }

    public Double getLatitude() {
        return latitude;
    }

    public void setLatitude(Double latitude) {
        this.latitude = latitude;
    }

    public String getSatelliteType() {
        return satelliteType;
    }

    public void setSatelliteType(String satelliteType) {
        this.satelliteType = satelliteType;
    }

    public String getLandType() {
        return landType;
    }

    public void setLandType(String landType) {
        this.landType = landType;
    }

    public String getConfidence() {
        return confidence;
    }

    public void setConfidence(String confidence) {
        this.confidence = confidence;
    }

    public String getFireImage() {
        return fireImage;
    }

    public void setFireImage(String fireImage) {
        this.fireImage = fireImage;
    }

    public String getSatelliteImage() {
        return satelliteImage;
    }

    public void setSatelliteImage(String satelliteImage) {
        this.satelliteImage = satelliteImage;
    }

    public String getAddTime() {
        return addTime;
    }

    public void setAddTime(String addTime) {
        this.addTime = addTime;
    }

    public String getFirePointAddress() {
        return firePointAddress;
    }

    public void setFirePointAddress(String firePointAddress) {
        this.firePointAddress = firePointAddress;
    }

    public String getRemark() {
        return remark;
    }

    public void setRemark(String remark) {
        this.remark = remark;
    }

    public String getPlaceHolder() {
        return placeHolder;
    }

    public void setPlaceHolder(String placeHolder) {
        this.placeHolder = placeHolder;
    }

    public String getProCode() {
        return proCode;
    }

    public void setProCode(String proCode) {
        this.proCode = proCode;
    }

    public String getProName() {
        return proName;
    }

    public void setProName(String proName) {
        this.proName = proName;
    }

    public String getCityCode() {
        return cityCode;
    }

    public void setCityCode(String cityCode) {
        this.cityCode = cityCode;
    }

    public String getCityName() {
        return cityName;
    }

    public void setCityName(String cityName) {
        this.cityName = cityName;
    }

    public String getCountyCode() {
        return countyCode;
    }

    public void setCountyCode(String countyCode) {
        this.countyCode = countyCode;
    }

    public String getCountyName() {
        return countyName;
    }

    public void setCountyName(String countyName) {
        this.countyName = countyName;
    }

    public String getTownCode() {
        return townCode;
    }

    public void setTownCode(String townCode) {
        this.townCode = townCode;
    }

    public String getTownName() {
        return townName;
    }

    public void setTownName(String townName) {
        this.townName = townName;
    }

    @Override
    public String toString() {
        return "FirePointPreciseEntity{" +
                "id=" + id +
                ", fireCode='" + fireCode + '\'' +
                ", satelliteTime='" + satelliteTime + '\'' +
                ", longitude=" + longitude +
                ", latitude=" + latitude +
                ", satelliteType='" + satelliteType + '\'' +
                ", landType='" + landType + '\'' +
                ", confidence='" + confidence + '\'' +
                ", fireImage='" + fireImage + '\'' +
                ", satelliteImage='" + satelliteImage + '\'' +
                ", addTime='" + addTime + '\'' +
                ", firePointAddress='" + firePointAddress + '\'' +
                ", remark='" + remark + '\'' +
                ", placeHolder='" + placeHolder + '\'' +
                ", proCode='" + proCode + '\'' +
                ", proName='" + proName + '\'' +
                ", cityCode='" + cityCode + '\'' +
                ", cityName='" + cityName + '\'' +
                ", countyCode='" + countyCode + '\'' +
                ", countyName='" + countyName + '\'' +
                ", townCode='" + townCode + '\'' +
                ", townName='" + townName + '\'' +
                '}';
    }
}
