package com.xkrs.straw.utils;

import com.xkrs.straw.dao.FirePointOrdinaryDao;
import com.xkrs.straw.dao.FirePointPreciseDao;
import com.xkrs.straw.model.bean.DataWrapper2;
import com.xkrs.straw.model.entity.FirePointOrdinaryEntity;
import com.xkrs.straw.model.entity.FirePointPreciseEntity;
import org.springframework.data.jpa.domain.Specification;

import javax.persistence.criteria.Predicate;
import java.util.ArrayList;
import java.util.List;

public class FirePointFilterUtils {

    private FirePointFilterUtils() {
    }

    public static DataWrapper2<Boolean, String> checkLandType(String landType) {
        if ("水体".equals(landType)) {
            return new DataWrapper2<>(Boolean.FALSE, "添加失败，不接收水体类型");
        }
        return new DataWrapper2<>(Boolean.TRUE, "");
    }

    public static DataWrapper2<Boolean, String> checkSatelliteType(String satelliteType, String confidence) {
        if ("Himawari 8".equals(satelliteType)) {
            if ("N".equals(confidence) || "L".equals(confidence)) {
                return new DataWrapper2<>(Boolean.FALSE, "添加失败，不接收葵花8中低置信度火点");
            }
        }
        return new DataWrapper2<>(Boolean.TRUE, "");
    }

    public static DataWrapper2<Boolean, String> checkDuplicated(FirePointOrdinaryDao firePointDao, FirePointOrdinaryEntity firePointEntity) {
        Specification<FirePointOrdinaryEntity> specification = (root, criteriaQuery, criteriaBuilder) -> {
            List<Predicate> predicateList = new ArrayList<>();
            predicateList.add(criteriaBuilder.equal(root.get("longitude").as(Double.class), firePointEntity.getLongitude()));
            predicateList.add(criteriaBuilder.equal(root.get("latitude").as(Double.class), firePointEntity.getLatitude()));
            predicateList.add(criteriaBuilder.equal(root.get("countyCode").as(String.class), firePointEntity.getCountyCode()));
            predicateList.add(criteriaBuilder.equal(root.get("countyName").as(String.class), firePointEntity.getCountyName()));
            predicateList.add(criteriaBuilder.equal(root.get("townCode").as(String.class), firePointEntity.getTownCode()));
            predicateList.add(criteriaBuilder.equal(root.get("satelliteTime").as(String.class), firePointEntity.getSatelliteTime()));
            predicateList.add(criteriaBuilder.equal(root.get("satelliteType").as(String.class), firePointEntity.getSatelliteType()));
            predicateList.add(criteriaBuilder.equal(root.get("landType").as(String.class), firePointEntity.getLandType()));
            return criteriaBuilder.and(predicateList.toArray(new Predicate[predicateList.size()]));
        };
        List<FirePointOrdinaryEntity> list = firePointDao.findAll(specification);
        if (list.size() > 0) {
            return new DataWrapper2<>(Boolean.TRUE, "该火点和历史火点重复，不入库");
        }
        return new DataWrapper2<>(Boolean.FALSE, "");
    }

    public static DataWrapper2<Boolean, String> checkDuplicated(FirePointPreciseDao firePointDao, FirePointPreciseEntity firePointEntity) {
        Specification<FirePointPreciseEntity> specification = (root, criteriaQuery, criteriaBuilder) -> {
            List<Predicate> predicateList = new ArrayList<>();
            predicateList.add(criteriaBuilder.equal(root.get("longitude").as(Double.class), firePointEntity.getLongitude()));
            predicateList.add(criteriaBuilder.equal(root.get("latitude").as(Double.class), firePointEntity.getLatitude()));
            predicateList.add(criteriaBuilder.equal(root.get("countyCode").as(String.class), firePointEntity.getCountyCode()));
            predicateList.add(criteriaBuilder.equal(root.get("countyName").as(String.class), firePointEntity.getCountyName()));
            predicateList.add(criteriaBuilder.equal(root.get("townCode").as(String.class), firePointEntity.getTownCode()));
            predicateList.add(criteriaBuilder.equal(root.get("satelliteTime").as(String.class), firePointEntity.getSatelliteTime()));
            predicateList.add(criteriaBuilder.equal(root.get("satelliteType").as(String.class), firePointEntity.getSatelliteType()));
            predicateList.add(criteriaBuilder.equal(root.get("landType").as(String.class), firePointEntity.getLandType()));
            return criteriaBuilder.and(predicateList.toArray(new Predicate[predicateList.size()]));
        };
        List<FirePointPreciseEntity> list = firePointDao.findAll(specification);
        if (list.size() > 0) {
            return new DataWrapper2<>(Boolean.TRUE, "该火点和历史火点重复，不入库");
        }
        return new DataWrapper2<>(Boolean.FALSE, "");
    }
}
