package com.xkrs.straw.utils;

import com.xkrs.straw.model.bean.VipLocalDateTimeRangePair;
import com.xkrs.straw.model.bean.VipTimeRangeBean;
import com.xkrs.utils.DateTimeUtils;
import org.apache.hc.core5.util.TextUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

public class VipTimeRangeUtils {

    public static Logger log = LoggerFactory.getLogger(VipTimeRangeUtils.class);

    private VipTimeRangeUtils() {
    }

    public static boolean checkIfInVipTimeRange(String vipTimeRangeJson) {
        return checkIfInVipTimeRange(vipTimeRangeJson, LocalDateTime.now());
    }

    /**
     * 校验传入的时间是否在指定的VIP时间范围内
     */
    public static boolean checkIfInVipTimeRange(String vipTimeRangeJson, LocalDateTime targetLocalDateTime) {
        try {
            if (TextUtils.isEmpty(vipTimeRangeJson) || targetLocalDateTime == null) {
                return false;
            }
            VipTimeRangeBean vipTimeRangeBean = JsonUtils.deserialize(vipTimeRangeJson, VipTimeRangeBean.class);
            if (vipTimeRangeBean == null || vipTimeRangeBean.getList() == null || vipTimeRangeBean.getList().size() == 0) {
                return false;
            }
            List<VipLocalDateTimeRangePair> vipLocalDateTimeRangePairList = obtainSortedVipLocalDateTimeRangePairList(vipTimeRangeBean.getList());
            if (vipLocalDateTimeRangePairList.size() == 0) {
                return false;
            }
            for (VipLocalDateTimeRangePair vipLocalDateTimeRangePair : vipLocalDateTimeRangePairList) {
                LocalDateTime startLocalDateTime = vipLocalDateTimeRangePair.getStartLocalDateTime();
                LocalDateTime endLocalDateTime = vipLocalDateTimeRangePair.getEndLocalDateTime();
                if (startLocalDateTime.isBefore(targetLocalDateTime) && endLocalDateTime.isAfter(targetLocalDateTime)) {
                    return true;
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.info("检查是否属于VIP时间范围时出错，详情=" + e.getMessage());
        }
        return false;
    }

    /**
     * 根据一个字符串格式时间的列表 生成 一个LocalDateTime格式时间的列表
     */
    private static List<VipLocalDateTimeRangePair> obtainSortedVipLocalDateTimeRangePairList(List<VipTimeRangeBean.VipTimeRangeItemBean> list) {
        List<VipLocalDateTimeRangePair> vipLocalDateTimeRangePairList = new ArrayList<>();
        try {
            if (list == null || list.size() == 0) {
                return vipLocalDateTimeRangePairList;
            }
            for (VipTimeRangeBean.VipTimeRangeItemBean vipTimeRangeItemBean : list) {
                String startTime = vipTimeRangeItemBean.getStartTime();
                LocalDateTime startLocalDateTime = LocalDateTime.parse(startTime, DateTimeUtils.DATE_TIME_FORMATTER_1);
                String endTime = vipTimeRangeItemBean.getEndTime();
                LocalDateTime endLocalDateTime = LocalDateTime.parse(endTime, DateTimeUtils.DATE_TIME_FORMATTER_1);

                VipLocalDateTimeRangePair vipLocalDateTimeRangePair = new VipLocalDateTimeRangePair();
                vipLocalDateTimeRangePair.setStartLocalDateTime(startLocalDateTime);
                vipLocalDateTimeRangePair.setEndLocalDateTime(endLocalDateTime);
                vipLocalDateTimeRangePairList.add(vipLocalDateTimeRangePair);
            }
            List<VipLocalDateTimeRangePair> sortedVipLocalDateTimeRangePairList = vipLocalDateTimeRangePairList.stream().sorted((o1, o2) -> {
                try {
                    LocalDateTime startLocalDateTime1 = o1.getStartLocalDateTime();
                    LocalDateTime endLocalDateTime1 = o1.getEndLocalDateTime();
                    LocalDateTime startLocalDateTime2 = o2.getStartLocalDateTime();
                    LocalDateTime endLocalDateTime2 = o2.getEndLocalDateTime();
                    if (startLocalDateTime1.isBefore(startLocalDateTime2) && endLocalDateTime1.isBefore(endLocalDateTime2)) {
                        return 1;
                    }
                    if (startLocalDateTime1.isAfter(startLocalDateTime2) && endLocalDateTime1.isAfter(endLocalDateTime2)) {
                        return -1;
                    }
                    return 0;
                } catch (Exception e) {
                    e.printStackTrace();
                }
                return 0;
            }).collect(Collectors.toList());
//            log.info("原始集合size=" + list.size() + "，格式化后的集合size=" + vipLocalDateTimeRangePairList.size() + "，排序后的集合size=" + sortedVipLocalDateTimeRangePairList.size());
            return sortedVipLocalDateTimeRangePairList;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return vipLocalDateTimeRangePairList;
    }

    public static String obtainLastOverTime(String oldVipTimeRangeJson) {
        //oldVipTimeRangeJson不为空说明不是新注册用户，解析该用户的VIP时间范围
        VipTimeRangeBean vipTimeRangeBean = JsonUtils.deserialize(oldVipTimeRangeJson, VipTimeRangeBean.class);
        if (vipTimeRangeBean == null || vipTimeRangeBean.getList() == null || vipTimeRangeBean.getList().size() == 0) {
            throw new RuntimeException("已保存的VIP时间范围反序列化失败");
        }
        List<VipTimeRangeBean.VipTimeRangeItemBean> list = vipTimeRangeBean.getList();
        if (list == null || list.size() == 0) {
            return "";
        }
        //校验数据合法性
        for (VipTimeRangeBean.VipTimeRangeItemBean vipTimeRangeItemBeanA : list) {
            for (VipTimeRangeBean.VipTimeRangeItemBean vipTimeRangeItemBeanB : list) {
                if (vipTimeRangeItemBeanA != vipTimeRangeItemBeanB) {
                    LocalDateTime startLocalDateTimeA = LocalDateTime.parse(vipTimeRangeItemBeanA.getStartTime(), DateTimeUtils.DATE_TIME_FORMATTER_1);
                    LocalDateTime endLocalDateTimeA = LocalDateTime.parse(vipTimeRangeItemBeanA.getEndTime(), DateTimeUtils.DATE_TIME_FORMATTER_1);
                    LocalDateTime startLocalDateTimeB = LocalDateTime.parse(vipTimeRangeItemBeanB.getStartTime(), DateTimeUtils.DATE_TIME_FORMATTER_1);
                    LocalDateTime endLocalDateTimeB = LocalDateTime.parse(vipTimeRangeItemBeanB.getEndTime(), DateTimeUtils.DATE_TIME_FORMATTER_1);
                    if (startLocalDateTimeB.isBefore(startLocalDateTimeA) && endLocalDateTimeB.isAfter(startLocalDateTimeA)) {
                        throw new RuntimeException("VIP开始结束时间存在交叉现象1，系统数据错误");
                    }
                    if (startLocalDateTimeB.isBefore(endLocalDateTimeA) && endLocalDateTimeB.isAfter(endLocalDateTimeA)) {
                        throw new RuntimeException("VIP开始结束时间存在交叉现象2，系统数据错误");
                    }
                }
            }
        }
        //获取结束时间最大的那个Pair
        VipTimeRangeBean.VipTimeRangeItemBean lastVipTimeRangeItemBean = list.get(0);
        for (VipTimeRangeBean.VipTimeRangeItemBean vipTimeRangeItemBean : list) {
            if (lastVipTimeRangeItemBean != vipTimeRangeItemBean) {
                LocalDateTime endLocalDateTime = LocalDateTime.parse(vipTimeRangeItemBean.getEndTime(), DateTimeUtils.DATE_TIME_FORMATTER_1);
                LocalDateTime lastEndLocalDateTime = LocalDateTime.parse(lastVipTimeRangeItemBean.getEndTime(), DateTimeUtils.DATE_TIME_FORMATTER_1);
                if (endLocalDateTime.isAfter(lastEndLocalDateTime)) {
                    lastVipTimeRangeItemBean = vipTimeRangeItemBean;
                }
            }
        }
        return lastVipTimeRangeItemBean.getEndTime();
    }


    /**
     * 生成一个新的 VipTimeRangeJson
     */
    public static String obtainNewVipTimeRangeJson(String oldVipTimeRangeJson, LocalDateTime newLocalDateTime) {
        //没有默认VIP时间范围的情况，是新注册用户
        if (TextUtils.isEmpty(oldVipTimeRangeJson)) {
            LocalDateTime startLocalDateTime = LocalDateTime.now();
            if (startLocalDateTime.isBefore(newLocalDateTime)) {
                VipTimeRangeBean.VipTimeRangeItemBean vipTimeRangeItemBean = new VipTimeRangeBean.VipTimeRangeItemBean();
                vipTimeRangeItemBean.setStartTime(DateTimeUtils.DATE_TIME_FORMATTER_1.format(startLocalDateTime));
                vipTimeRangeItemBean.setEndTime(DateTimeUtils.DATE_TIME_FORMATTER_1.format(newLocalDateTime));
                List<VipTimeRangeBean.VipTimeRangeItemBean> list = new ArrayList<>();
                list.add(vipTimeRangeItemBean);
                VipTimeRangeBean vipTimeRangeBean = new VipTimeRangeBean();
                vipTimeRangeBean.setList(list);
                return JsonUtils.<VipTimeRangeBean>serialize(vipTimeRangeBean);
            } else {
                throw new RuntimeException("初始化的过期时间必须晚于当前时间");
            }
        }
        //oldVipTimeRangeJson不为空说明不是新注册用户，解析该用户的VIP时间范围
        VipTimeRangeBean vipTimeRangeBean = JsonUtils.deserialize(oldVipTimeRangeJson, VipTimeRangeBean.class);
        if (vipTimeRangeBean == null || vipTimeRangeBean.getList() == null || vipTimeRangeBean.getList().size() == 0) {
            throw new RuntimeException("已保存的VIP时间范围反序列化失败");
        }
        List<VipTimeRangeBean.VipTimeRangeItemBean> list = vipTimeRangeBean.getList();
        //校验数据合法性
        for (VipTimeRangeBean.VipTimeRangeItemBean vipTimeRangeItemBeanA : list) {
            for (VipTimeRangeBean.VipTimeRangeItemBean vipTimeRangeItemBeanB : list) {
                if (vipTimeRangeItemBeanA != vipTimeRangeItemBeanB) {
                    LocalDateTime startLocalDateTimeA = LocalDateTime.parse(vipTimeRangeItemBeanA.getStartTime(), DateTimeUtils.DATE_TIME_FORMATTER_1);
                    LocalDateTime endLocalDateTimeA = LocalDateTime.parse(vipTimeRangeItemBeanA.getEndTime(), DateTimeUtils.DATE_TIME_FORMATTER_1);
                    LocalDateTime startLocalDateTimeB = LocalDateTime.parse(vipTimeRangeItemBeanB.getStartTime(), DateTimeUtils.DATE_TIME_FORMATTER_1);
                    LocalDateTime endLocalDateTimeB = LocalDateTime.parse(vipTimeRangeItemBeanB.getEndTime(), DateTimeUtils.DATE_TIME_FORMATTER_1);
                    if (startLocalDateTimeB.isBefore(startLocalDateTimeA) && endLocalDateTimeB.isAfter(startLocalDateTimeA)) {
                        throw new RuntimeException("VIP开始结束时间存在交叉现象1，系统数据错误");
                    }
                    if (startLocalDateTimeB.isBefore(endLocalDateTimeA) && endLocalDateTimeB.isAfter(endLocalDateTimeA)) {
                        throw new RuntimeException("VIP开始结束时间存在交叉现象2，系统数据错误");
                    }
                }
            }
        }
        //获取结束时间最大的那个Pair
        VipTimeRangeBean.VipTimeRangeItemBean lastVipTimeRangeItemBean = list.get(0);
        for (VipTimeRangeBean.VipTimeRangeItemBean vipTimeRangeItemBean : list) {
            if (lastVipTimeRangeItemBean != vipTimeRangeItemBean) {
                LocalDateTime endLocalDateTime = LocalDateTime.parse(vipTimeRangeItemBean.getEndTime(), DateTimeUtils.DATE_TIME_FORMATTER_1);
                LocalDateTime lastEndLocalDateTime = LocalDateTime.parse(lastVipTimeRangeItemBean.getEndTime(), DateTimeUtils.DATE_TIME_FORMATTER_1);
                if (endLocalDateTime.isAfter(lastEndLocalDateTime)) {
                    lastVipTimeRangeItemBean = vipTimeRangeItemBean;
                }
            }
        }
        //获取到结束时间最大的那个Pair，用它的结束时间和当前时间做比较
        LocalDateTime lastEndLocalDateTime = LocalDateTime.parse(lastVipTimeRangeItemBean.getEndTime(), DateTimeUtils.DATE_TIME_FORMATTER_1);
        if (lastEndLocalDateTime.isBefore(LocalDateTime.now())) {
            //最大结束时间早于当前时间，说明该用户已过期，是普通用户。
            //调整普通用户过期时间的方案是添加新的Pair，开始时间是当前时间，结束时间是传入的时间
            LocalDateTime startLocalDateTime = LocalDateTime.now();
            if (startLocalDateTime.isBefore(newLocalDateTime)) {
                VipTimeRangeBean.VipTimeRangeItemBean vipTimeRangeItemBean = new VipTimeRangeBean.VipTimeRangeItemBean();
                vipTimeRangeItemBean.setStartTime(DateTimeUtils.DATE_TIME_FORMATTER_1.format(startLocalDateTime));
                vipTimeRangeItemBean.setEndTime(DateTimeUtils.DATE_TIME_FORMATTER_1.format(newLocalDateTime));
                list.add(vipTimeRangeItemBean);
                return JsonUtils.<VipTimeRangeBean>serialize(vipTimeRangeBean);
            } else {
                throw new RuntimeException("为断更用户添加的过期时间必须晚于当前时间");
            }
        } else {
            //最大结束时间晚于当前时间，说明该用户未过期，是VIP用户。
            //调整VIP用户过期时间的方案是调整结束时间为传入的时间
            LocalDateTime startLocalDateTime = LocalDateTime.parse(lastVipTimeRangeItemBean.getStartTime(), DateTimeUtils.DATE_TIME_FORMATTER_1);
            if (startLocalDateTime.isBefore(newLocalDateTime)) {
                lastVipTimeRangeItemBean.setEndTime(DateTimeUtils.DATE_TIME_FORMATTER_1.format(newLocalDateTime));
                return JsonUtils.<VipTimeRangeBean>serialize(vipTimeRangeBean);
            } else {
                throw new RuntimeException("设置的过期时间必须晚于当次的VIP开始时间");
            }
        }
    }

    public static void main(String[] args) {
        VipTimeRangeBean.VipTimeRangeItemBean vipTimeRangeItemBean1 = new VipTimeRangeBean.VipTimeRangeItemBean();
        vipTimeRangeItemBean1.setStartTime("2022-06-27 11:15:00");
        vipTimeRangeItemBean1.setEndTime("2022-07-07 18:20:00");
        VipTimeRangeBean.VipTimeRangeItemBean vipTimeRangeItemBean2 = new VipTimeRangeBean.VipTimeRangeItemBean();
        vipTimeRangeItemBean2.setStartTime("2022-07-09 18:20:00");
        vipTimeRangeItemBean2.setEndTime("2022-07-20 18:20:00");
        List<VipTimeRangeBean.VipTimeRangeItemBean> list = new ArrayList<>();
        list.add(vipTimeRangeItemBean1);
        list.add(vipTimeRangeItemBean2);
        VipTimeRangeBean vipTimeRangeBean = new VipTimeRangeBean();
        vipTimeRangeBean.setList(list);
        String serialize = JsonUtils.<VipTimeRangeBean>serialize(vipTimeRangeBean);
        System.out.println("serialize = " + serialize);
    }

}
