package com.xkrs.utils;

import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpRequestBase;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * http工具类
 */
public class WDHttpClientUtils {

    private static final boolean printLog = false;

    private static final Logger log = LoggerFactory.getLogger(HttpClientUtils.class);

    /**
     * 发送http+get请求
     *
     * @param url
     * @return 返回结果
     * @throws Exception
     */
    public static String sendHttpGet(String url) throws Exception {
        CloseableHttpClient httpClient = HttpClients.createDefault();
        return doGet(url, httpClient);
    }

    /**
     * 发送http+post请求
     *
     * @param url    请求地址
     * @param params 请求参数 json字符串
     * @return 返回结果
     * @throws Exception
     */
    public static String sendHttpPost(String url, String params) throws Exception {
        CloseableHttpClient httpClient = HttpClients.createDefault();
        return doPost(httpClient, url, params);
    }

    /**
     * 发送http+post请求
     *
     * @param url 请求地址
     * @return 返回结果
     * @throws Exception
     */
    public static String sendHttpPost(String url) throws Exception {
        CloseableHttpClient httpClient = HttpClients.createDefault();
        return doPost(httpClient, url);
    }

    /**
     * 封装get请求方式的处理
     */
    private static String doGet(String url, CloseableHttpClient httpClient) throws Exception {
        if (printLog) {
            log.info("HGet请求url={}", url);
        }
        HttpGet httpGet = new HttpGet(url);
        return execute(httpClient, httpGet);
    }

    /**
     * 封装post请求方式的处理
     */
    private static String doPost(CloseableHttpClient httpClient, String url, String params) throws Exception {
        if (printLog) {
            log.info("Post请求url：{}", url);
            log.info("Post请求params：{}", params);
        }
        HttpPost httpPost = new HttpPost(url);
        httpPost.addHeader("Content-Type", "application/json; charset=utf-8");
        httpPost.setEntity(new StringEntity(params, "utf-8"));
        return execute(httpClient, httpPost);
    }

    /**
     * 封装post请求方式的处理
     */
    private static String doPost(CloseableHttpClient httpClient, String url) throws Exception {
        if (printLog) {
            log.info("Post请求url：{}", url);
        }
        HttpPost httpPost = new HttpPost(url);
        httpPost.addHeader("Content-Type", "application/json; charset=utf-8");
        return execute(httpClient, httpPost);
    }

    /**
     * 执行发送
     */
    private static String execute(CloseableHttpClient httpClient, HttpRequestBase requestBase) throws Exception {
        String result = null;
        CloseableHttpResponse response = null;
        try {
            response = httpClient.execute(requestBase);
            int statusCode = response.getStatusLine().getStatusCode();
            if (printLog) {
                log.info("HttpClient响应码={}", statusCode);
            }
            final int success = 200;
            if (statusCode == success) {
                result = EntityUtils.toString(response.getEntity(), "utf-8");
            } else {
                if (printLog) {
                    log.error("HttpClient请求失败，错误码={}", statusCode);
                }
            }
        } catch (Exception e) {
            if (printLog) {
                log.error("HttpClient请求异常：", e);
            }
        } finally {
            if (null != httpClient) {
                httpClient.close();
            }
            if (null != response) {
                response.close();
            }
        }
        if (printLog) {
            log.info("HttpClient请求结果：{}", result);
        }
        return result;
    }

}


